import { ParseResultType, fromUrl, parseDomain } from 'parse-domain';
import Cloudflare from 'cloudflare';
import IPUtils from './ip-utils.js';
const ipv4Regex = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/u;
const ipv6Regex = /^(?:(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){6})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:::(?:(?:(?:[0-9a-fA-F]{1,4})):){5})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:(?:[0-9a-fA-F]{1,4})):){4})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,1}(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:(?:[0-9a-fA-F]{1,4})):){3})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,2}(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:(?:[0-9a-fA-F]{1,4})):){2})(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,3}(?:(?:[0-9a-fA-F]{1,4})))?::(?:(?:[0-9a-fA-F]{1,4})):)(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,4}(?:(?:[0-9a-fA-F]{1,4})))?::)(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9]))\.){3}(?:(?:25[0-5]|(?:[1-9]|1[0-9]|2[0-4])?[0-9])))))))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,5}(?:(?:[0-9a-fA-F]{1,4})))?::)(?:(?:[0-9a-fA-F]{1,4})))|(?:(?:(?:(?:(?:(?:[0-9a-fA-F]{1,4})):){0,6}(?:(?:[0-9a-fA-F]{1,4})))?::))))$/u;
export default class CloudflareClient {
    cloudflare;
    zoneMap = new Map();
    constructor(cloudflareOptions) {
        this.cloudflare = new Cloudflare(cloudflareOptions);
        this.updateZoneMap();
    }
    async syncRecord(record, ip) {
        const recordIds = await this.getRecordIdsForRecords([record]);
        const ipToUse = ip ? ip : await IPUtils.getIpv4();
        const zoneId = await this.getZoneIdByRecordName(record.name);
        const recordId = recordIds.get(this.getRecordIdMapKey(record));
        const recordExists = recordId !== undefined;
        if (recordExists) {
            const result = await this.updateRecord(zoneId, recordId, record, ipToUse);
            return result;
        }
        const result = await this.createRecord(zoneId, record, ipToUse);
        return result;
    }
    async syncRecords(records, ip) {
        const recordIds = await this.getRecordIdsForRecords(records);
        const ipToUse = ip ? ip : await IPUtils.getIpv4();
        const resultPromises = records.map(async (record) => {
            const zoneId = await this.getZoneIdByRecordName(record.name);
            const recordId = recordIds.get(this.getRecordIdMapKey(record));
            const recordExists = recordId !== undefined;
            if (recordExists) {
                const currentResult = await this.updateRecord(zoneId, recordId, record, ipToUse);
                return currentResult;
            }
            const currentResult = await this.createRecord(zoneId, record, ipToUse);
            return currentResult;
        });
        const results = await Promise.all(resultPromises);
        return results;
    }
    async removeRecordByNameAndType(recordName, recordType) {
        const recordTypeToUse = recordType ? recordType : 'A';
        const zoneId = await this.getZoneIdByRecordName(recordName);
        const recordId = await this.getRecordIdByNameAndType(recordName, recordTypeToUse);
        await this.cloudflare.dns.records.delete(recordId, { zone_id: zoneId });
    }
    async getRecordDataForRecord(record) {
        const domain = this.getDomainByRecordName(record.name);
        const recordDataForDomain = await this.getRecordsByDomain(domain);
        const recordData = recordDataForDomain.find((singleRecordData) => record.name.toLowerCase() === singleRecordData.name.toLowerCase());
        return recordData;
    }
    async getRecordDataForRecords(records) {
        const domains = this.getDomainsFromRecords(records);
        const recordDataPromises = domains.map(async (domain) => {
            const recordDataForDomain = await this.getRecordsByDomain(domain);
            const recordDataForDomainFilteredByRecords = recordDataForDomain.filter((singleRecordData) => records.some((record) => record.name.toLowerCase() === singleRecordData.name.toLowerCase()));
            return recordDataForDomainFilteredByRecords;
        });
        const recordDataForDomains = await Promise.all(recordDataPromises);
        const recordData = [].concat(...recordDataForDomains);
        return recordData;
    }
    async getRecordsByDomains(domains) {
        const recordDataPromises = domains.map((domain) => this.getRecordsByDomain(domain));
        const recordDataForDomains = await Promise.all(recordDataPromises);
        const recordData = {};
        recordDataForDomains.forEach((recordDataForDomain, index) => {
            recordData[domains[index]] = recordDataForDomain;
        });
        return recordData;
    }
    async getRecordsByDomain(domain) {
        const zoneId = await this.getZoneIdByDomain(domain);
        const allRecords = [];
        for await (const recordListResponse of this.cloudflare.dns.records.list({ zone_id: zoneId })) {
            allRecords.push(recordListResponse);
        }
        return allRecords;
    }
    async createRecord(zoneId, recordData, ip) {
        const dnsRecordToCreate = {
            ...recordData,
            name: recordData.name.toLowerCase(),
            content: (recordData.content ? recordData.content : ip),
            type: (recordData.type ? recordData.type : 'A'),
            ttl: recordData.ttl ? recordData.ttl : 1,
            zone_id: zoneId,
        };
        if (!dnsRecordToCreate.content) {
            throw Error(`Could not create Record "${dnsRecordToCreate.name}": Content is missing!`);
        }
        if (dnsRecordToCreate.type === 'A') {
            if (!dnsRecordToCreate.content.match(ipv4Regex)) {
                throw Error(`Could not create Record "${dnsRecordToCreate.name}": '${dnsRecordToCreate.content}' is not a valid ipv4!`);
            }
        }
        else if (dnsRecordToCreate.type === 'AAAA') {
            if (!dnsRecordToCreate.content.match(ipv6Regex)) {
                throw Error(`Could not create Record "${dnsRecordToCreate.name}": '${dnsRecordToCreate.content}' is not a valid ipv6!`);
            }
        }
        else if (dnsRecordToCreate.type === 'CNAME') {
            const parsedDomain = parseDomain(fromUrl(dnsRecordToCreate.content));
            if (parsedDomain.type !== ParseResultType.Listed || !parsedDomain.domain) {
                throw Error(`Could not create Record "${dnsRecordToCreate.name}": '${dnsRecordToCreate.content}' is not a valid domain name!`);
            }
        }
        return this.cloudflare.dns.records.create(dnsRecordToCreate);
    }
    async updateRecord(zoneId, recordId, recordData, ip) {
        const updatedDnsRecord = {
            ...recordData,
            name: recordData.name.toLowerCase(),
            content: (recordData.content ? recordData.content : ip),
            type: (recordData.type ? recordData.type : 'A'),
            ttl: recordData.ttl ? recordData.ttl : 1,
            zone_id: zoneId,
        };
        if (!updatedDnsRecord.content) {
            throw Error(`Could not update Record "${updatedDnsRecord.name}": Content is missing!`);
        }
        if (updatedDnsRecord.type === 'A') {
            if (!updatedDnsRecord.content.match(ipv4Regex)) {
                throw Error(`Could not update Record "${updatedDnsRecord.name}": '${updatedDnsRecord.content}' is not a valid ipv4!`);
            }
        }
        else if (updatedDnsRecord.type === 'AAAA') {
            if (!updatedDnsRecord.content.match(ipv6Regex)) {
                throw Error(`Could not update Record "${updatedDnsRecord.name}": '${updatedDnsRecord.content}' is not a valid ipv6!`);
            }
        }
        else if (updatedDnsRecord.type === 'CNAME') {
            const parsedDomain = parseDomain(fromUrl(updatedDnsRecord.content));
            if (parsedDomain.type !== ParseResultType.Listed || !parsedDomain.domain) {
                throw Error(`Could not update Record "${updatedDnsRecord.name}": '${updatedDnsRecord.content}' is not a valid domain name!`);
            }
        }
        return await this.cloudflare.dns.records.edit(recordId, updatedDnsRecord);
    }
    async updateZoneMap() {
        const response = await this.cloudflare.zones.list();
        let zones = response.result;
        let nextPageExists = response.hasNextPage();
        while (nextPageExists) {
            const nextPageResponse = await response.getNextPage();
            zones = zones.concat(nextPageResponse.result);
            nextPageExists = nextPageResponse.hasNextPage();
        }
        this.zoneMap = new Map();
        for (const zone of zones) {
            this.zoneMap.set(zone.name, zone.id);
        }
    }
    async getRecordIdByNameAndType(recordName, recordType) {
        const record = await this.getRecordByNameAndType(recordName, recordType);
        return record.id;
    }
    getZoneIdByRecordName(recordName) {
        const domain = this.getDomainByRecordName(recordName);
        return this.getZoneIdByDomain(domain);
    }
    async getRecordByNameAndType(recordName, recordType) {
        const domain = this.getDomainByRecordName(recordName);
        const records = await this.getRecordsByDomain(domain);
        const record = records.find((currentRecord) => currentRecord.name.toLowerCase() === recordName.toLowerCase() && currentRecord.type.toLowerCase() === recordType.toLowerCase());
        const recordNotFound = record === undefined;
        if (recordNotFound) {
            throw new Error(`Record '${recordName}' not found.`);
        }
        return record;
    }
    async getRecordIdsForRecords(records) {
        const recordIdMap = new Map();
        const recordData = await this.getRecordDataForRecords(records);
        for (const record of recordData) {
            recordIdMap.set(this.getRecordIdMapKey(record), record.id);
        }
        return recordIdMap;
    }
    getRecordIdMapKey(record) {
        const recordName = record.name.toLowerCase();
        const recordType = record.type ? record.type.toLowerCase() : 'a';
        return `"${recordName}"_"${recordType}"`;
    }
    async getZoneIdByDomain(domain) {
        if (this.zoneMap.has(domain)) {
            const zoneId = this.zoneMap.get(domain.toLowerCase());
            return zoneId;
        }
        await this.updateZoneMap();
        if (!this.zoneMap.has(domain)) {
            throw new Error(`Could not find domain '${domain}'. Make sure the domain is set up for your cloudflare account.`);
        }
        const zoneId = this.zoneMap.get(domain.toLowerCase());
        return zoneId;
    }
    getDomainsFromRecords(records) {
        const domains = records.map((record) => this.getDomainByRecordName(record.name)).filter((domain, index, domainList) => domainList.indexOf(domain.toLowerCase()) === index);
        return domains;
    }
    getDomainByRecordName(recordName) {
        const parsedDomain = parseDomain(fromUrl(recordName));
        if (parsedDomain.type !== ParseResultType.Listed || !parsedDomain.domain) {
            throw new Error(`Could not parse domain. '${JSON.stringify(recordName)}' is not a valid record name.`);
        }
        let domain = '';
        domain += parsedDomain.domain;
        for (const tld of parsedDomain.topLevelDomains) {
            domain += `.${tld}`;
        }
        return domain.toLowerCase();
    }
}
